// Ex0702: フォトリフレクタ(正規化，最大，最小), LED表示
// 実行時間を変更してみる
#include <FastLED.h>

// LED 16個
const int DATA_PIN = 19; // LEDの接続ピン
const int NUM_LEDS = 16; // LEDの個数
CRGB leds[NUM_LEDS]; // LED用の配列

#define NUM_PR  5 // センサの数
const int pr_pins[NUM_PR] = {A3, A6, A7, A4, A5}; // ピン定義
int pr[NUM_PR] = {0}; // センサの値
int pr_bin[NUM_PR] = {0}; // ラインの有無(0,1)
int pr_min[NUM_PR] = {4095, 4095, 4095, 4095, 4095}; // 最小値
int pr_max[NUM_PR] = {0}; // 最大値
float pr_nrm[NUM_PR] = {0}; // 正規化値 0.0-1.0
unsigned long pr_time = 0; // フォトインタラプタ用タイマー計測値

void setup() {
  Serial.begin(115200);
  delay(100);

  // led
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS);
  FastLED.setBrightness(32); // 最大の明るさ 0-255
}

void loop() {
  if (millis() - pr_time > 10) { // 10ms毎に実行
    pr_time = millis();
    get_pr(); // センサの値取得，閾値処理
    led_pos(); // センサの反応位置を点灯
    // 実行結果の表示
//    pr_print_bin(); // 65ms
  }
}

// フォトリフレクタの反応位置にあわせてLEDを点灯
void led_pos() {
  for (int i = 0; i < NUM_PR; i++) {
    if (pr_bin[i] == 1) {
      leds[i+10] = CRGB(255, 0, 0); // 赤
    } else {
      leds[i+10] = CRGB(0, 0, 0); // 消灯
    }
  }
  FastLED.show(); // 表示を更新
}

// フォトリフレクタの値を読み込み，最小・最大を調べて閾値処理
void get_pr() {
  // センサの値を取得
  for (int i = 0; i < NUM_PR; i++) {
    pr[i] = analogRead(pr_pins[i]);
  }
  // 最小，最大
  for (int i = 0; i < NUM_PR; i++) {
    if (pr[i] < pr_min[i]) pr_min[i] = pr[i];
    if (pr[i] > pr_max[i]) pr_max[i] = pr[i];
  }
  // 閾値処理
  for (int i = 0; i < NUM_PR; i++) {
    pr_nrm[i] = (pr[i] - pr_min[i]) / (float)(pr_max[i] - pr_min[i]);
    if (pr_nrm[i] > 0.6) pr_bin[i] = 1;
    else pr_bin[i] = 0;
  }
}

// フォトリフレクタの状態(0,1)表示
void pr_print_bin() {
  for (int i = 0; i < NUM_PR; i++) {
    Serial.print(pr_bin[i]);
    if (i < NUM_PR - 1) Serial.print(",");
  }
  Serial.println();
}
